/*
 * INFRA RED REMOTE CONTROLER DECODER FOR THE GRANDIN REMOTE
 * USING THE RC5 CODE
 * Copyright Tristus1er and Valour Inc. 2003
 * V1.0 March 2003
 * Made at ESSI SCHOOL at Sophia-Antipolis (06) France for the STREAM training
 */

#include <stdio.h>
#include <fcntl.h> 

#define LITTLE_MIN 7
#define LITTLE_MAX 10
#define BIG_MIN 17
#define BIG_MAX 20

#define BUFFER_SIZE 28
#define TRAME_SIZE 14

#define HEADER_SIZE 3
#define ADDRESS_SIZE 5
#define COMMAND_SIZE 6

char buffer_logic[TRAME_SIZE];

/***********************************************
 *           Function print_equipment          *
 * --------------------------------------------*
 * This function will give the type of         *
 * equipment                                   *
 *                                             *
 * args int the number of the equipment type   *
 *                                             *
 ***********************************************/
void print_equipment(int device_type) {

  if (device_type > 31 || device_type < 0) {
    printf("ERROR, not a valid device");
  }

switch (device_type)
    {
    case 0 : printf("TV SET 1");
      break;      
    case 1 :  printf("TV SET 2");
      break;      
    case 2 :  printf("VIDEOTEXT");
      break;      
    case 3 :  printf("EXPANSION FOR TV 1 AND 2");
      break;      
    case 4 :  printf("LASER VIDEO PLAYER");
      break;      
    case 5 :  printf("VIDEO RECORDER 1 (VCR 1)");
      break;
    case 6 :  printf("VIDEO RECORDER 2 (VCR 2)");
      break;
    case 7 :  printf("RESERVED");
      break;      
    case 8 :  printf("SAT 1");
      break;      
    case 9 :  printf("EXPANSION FOR VCR 1 OR 2");
      break;
    case 10 :  printf("SAT 2");
      break;
    case 11 :  printf("RESERVED");
      break;
    case 12 :  printf("CD VIDEO");
      break;      
    case 13 :  printf("RESERVED");
      break;      
    case 14 :  printf("CD PHOTO");
      break;      
    case 15 :  printf("RESERVED");
      break;
    case 16 :  printf("AUDIO PREAMPLIFIER 1");
      break;      
    case 17 :  printf("RECEIVER / TUNER");
      break;      
    case 18 :  printf("TAPE / CASSETE RECORDER");
      break;      
    case 19 :  printf("AUDIO PREAMPLIFIER 2");
      break;      
    case 20 :  printf("CD");
      break;      
    case 21 :  printf("AUDIO RACK");
      break;   
    case 22 :  printf("AUDIO SAT RECEIVER");
      break;      
    case 23 :  printf("DCC RECORDER");
      break;      
    case 24 :  printf("RESERVED");
      break;   
    case 25 :  printf("RESERVED");
      break;   
    case 26 :  printf("WRITABLE CD");
      break;   
    case 27 :  printf("RESERVED");
      break;   
    case 28 :  printf("RESERVED");
      break;   
    case 29 :  printf("RESERVED");
      break;   
    default: printf("RESERVED"); // for  26-31
      break;
    } // end switch
}// end print_equipment



/***********************************************
 *           Function print_command            *
 * --------------------------------------------*
 * This function will give a meaning           *
 * of the command                              *
 *                                             *
 * args int the number of the command          *
 *                                             *
 ***********************************************/
void print_command(int command_number) {

  if (command_number > 124 || command_number < 0) {
    printf("ERROR, not a valid command");
  }

switch(command_number)
    {
    case 0 : printf("0");
      break;
    case 1 : printf("1");
      break;
    case 2 : printf("2");
      break;
    case 3 : printf("3");
      break;
    case 4 : printf("4");
      break;
    case 5 : printf("5");
      break;
    case 6 : printf("6");
      break;
    case 7 : printf("7");
      break;
    case 8 : printf("8");
      break;
    case 9 : printf("9");
      break;
      // added for the GRANDIN remote
    case 10 : printf("--");
      break;
      // end added for the GRANDIN remote

    case 12 : printf("STANDBY");
      break;
    case 13 : printf("MUTE");
      break;
    case 14 : printf("PRESETS");
      break;
    case 16 : printf("VOLUME UP");
      break;
    case 17 : printf("VOLUME DOWN");
      break;

      // modified for the GRANDIN remote
      // case 18 : printf("BRIGHTNESS +");
      // break;
    case 18 : printf("INFO");
      break;

    case 19 : printf("BRIGHTNESS -");
      break;
    case 20 : printf("COLOR SATURATION +");
      break;

      // modified for the GRANDIN remote
      // case 21 : printf("COLOR SATURATION -");
      // break;
      // case 22 : printf("BASS UP");
      // break;
    case 21 : printf("MENU LEFT");
      break;
    case 22 : printf("MENU RIGHT");
      break;

    case 23 : printf("BASS DOWN");
      break;
    case 24 : printf("TREBLE +");
      break;
    case 25 : printf("TREBLE -");
      break;
    case 26 : printf("BALANCE RIGHT");
      break;
    case 27 : printf("BALANCE LEFT");
      break;

      // added for the GRANDIN remote
    case 32 : printf("P+ / MENU UP");
      break;
    case 33 : printf("P- / MENU DOWN");
      break;
    case 34 : printf("P<P");
      break;
    case 35 : printf("I-II");
      break;
    case 47 : printf("\n\nThanks for using our program\n\n");
      exit(0);
      break;      
      // end added for the GRANDIN remote
      
      // modified for the GRANDIN remote
      // case 48 : printf("PAUSE");
      // break;
      // case 50 : printf("FAST REVERSE");
      // break;
      // case 52 : printf("FAST FORWARD-");
      // break;
      //case 53 : printf("PLAY");
      //break;
      // case 54 : printf("STOP");
      // break;
      // case 55 : printf("RECORD");
      // break;

    case 48 : printf("M");
      break;
    case 50 : printf("FEATURE");
      break;
    case 52 : printf("INSTALL");
      break;
    case 54 : printf("PICTURE");
      break;
    case 55 : printf("SOUND");
      break;
    case 63 : printf("SYSTEM SELECT");
      break;
    case 71 : printf("DIM LOCAL DISPLAY");
      break;
    case 77 : printf("LINEAR FUNCTION (+)");
      break;
    case 78 : printf("LINEAR FUNCTION (-)");
      break;
    case 80 : printf("STEP UP");
      break;
    case 81 : printf("STEP DOWN");
      break;
    case 82 : printf("MENU ON");
      break;
    case 83 : printf("MENU OFF");
      break;
    case 84 : printf("DISPLAY A/V SYS STATUS");
      break;
    case 85 : printf("STEP LEFT");
      break;
    case 86 : printf("STEP RIGHT");
      break;
    case 87 : printf("ACKNOWLEDGE");
      break;
    case 88 : printf("PIP ON/OFF");
      break;
    case 89 : printf("PIP SHIFT");
      break;
    case 90 : printf("PIP MAIN SWAP");
      break;
    case 91 : printf("STROBE ON/OFF");
      break;
    case 92 : printf("MULTI STROBE");
      break;
    case 93 : printf("MAIN FROZEN");
      break;
    case 94 : printf("3/9 MULTI SCAN");
      break;
    case 95 : printf("PIP SELECT");
      break;
    case 96 : printf("MOSAIC MULTI PIP");
      break;
    case 97 : printf("PICTURE DNR");
      break;
    case 98 : printf("MAIN STORED");
      break;
    case 99 : printf("PIP STROBE");
      break;
    case 100 : printf("RECALL MAIN PICTURE");
      break;
    case 101 : printf("PIP FREEZE");
      break;
    case 102 : printf("PIP STEP UP");
      break;
    case 103 : printf("PIP STEP DOWN");
      break;
    case 118 : printf("SUB MODE");
      break;
    case 119 : printf("OPTIONS BUS MODE");
      break;
    case 123 : printf("CONNECT");
      break;
    case 124 : printf("DISCONNECT");
      break;
    default: printf("UNKNOWN");
      break;
    } // end switch

} // end print_command


/***********************************************
 *      Function decode_binphilips2values      *
 * --------------------------------------------*
 *                                             *
 * args char* the array that contain the datas *
 *                                             *
 ***********************************************/
void decode_binphilips2values(char *input){
  int i;
  int buffer_pos;
  int address = 0;
  int command = 0;


  buffer_pos = 0;

  // read the header (but i don't keep the value
  for(i = 0; i<HEADER_SIZE; i++ ){
    *input++;
  }// end for header

  // read the address part and keep it in var address
  for(i = 0; i<ADDRESS_SIZE; i++ ){
    address  *=2;
    address += (*input++);
  }// end for address

  // read the command part and keep it in var command
  for(i = 0; i<COMMAND_SIZE; i++ ){
    command *=2;
    command += (*input++);   
  } // end for command
  
  // print the values readed
  printf("address[%d]", address);
  print_equipment(address);
  printf(" => ");

  printf("command[%d] : ", command);
  print_command(command);
  printf("\n");

}// end decode_binphilips2values

/***********************************************
 *           Function print_equipment          *
 * --------------------------------------------*
 * This function will give the type of         *
 * equipment                                   *
 *                                             *
 * args char* the array that contain the values*
 * args int the size of the array              *
 *                                             *
 ***********************************************/
void print_buffer(char *buffer, int size){
  // just a loop to display the buffer filled with 0 and 0
  int i;
  for (i=0; i<size; i++){
    if (buffer[i] == 0){printf("0");}
    else {printf("1");}
  } // end for
} // end print_buffer


/***********************************************
 *         Function decode_manchester2bin      *
 * --------------------------------------------*
 * This function will convert the physical     *
 * 1 and 0 into logical 1 and 0 using the      *
 * manchester coding                           *
 *                                             *
 * args char* the array that contain           *
 *            the physical values              *
 * returns char* the array that contain        *
 *               the logical values            *
 *                                             *
 ***********************************************/
char* decode_manchester2bin(char buffer_in[BUFFER_SIZE]){
  int buffer_position;
  int i;
  
  buffer_position = 0;

  // take only the even values
  // the odd values must be the opposite value of the even value
  for (i=0; i<BUFFER_SIZE ; i++){
    if (i%2 == 0) {
      buffer_logic[buffer_position++]=buffer_in[i];
    } // end if    
  }// end for

  return buffer_logic;

} // end decode_manchester2bin



int main(){
	int fd;

	int cont;
	int pointeur_buffer;
	int state = 1;

	char buffer[BUFFER_SIZE];


	printf("\n");
	printf("\n");
	printf("IR REMOTE DECODER FOR GRANDIN REMOTE\n");
	printf("By Tristan SALAUN and Delphine VALOUR\n");
	printf("   ESSI 3 - STREAM - March 2003\n");
	printf("\n");
	printf("\n");
	printf("Statting decoding\n");
	
	// open the Real Time Fifo number 0 to read the values of the RTAI module
	fd=open("/dev/rtf0",O_RDONLY);

	pointeur_buffer = 1;

	// infinite loop
	while (1)
	  {	  
	    // read the value in the FIFO
	    read(fd,&cont,sizeof(cont));

	    // print the value returned by the module
	    //printf("%d duree %d",state, cont);
	    
	    if ( LITTLE_MAX >= cont && cont >= LITTLE_MIN) 
		  { 
		    // if it's a small value (here about 900us, 899us to be exact)
		    // then write the value of the current state in the buffer
		    // printf(" Small value\n");
		    buffer[pointeur_buffer++] = state;
		  }
		if ( BIG_MAX >= cont && cont >= BIG_MIN ) 
		  { 
		    // if it's a big value (double of the small one)
		    // then write twice the value of the current state in the buffer
		    //printf(" >>>>> BIG VALUE\n");
		    buffer[pointeur_buffer++] = state;
		    buffer[pointeur_buffer++] = state;
		  } // end  ( BIG_MAX >= cont && cont >= BIG_MIN )
		
		
		if ( cont == 30 ) {
		  // if the module return the value 30 it mean that it's a end of a frame
		  //printf("******** END FRAME *************\n");
		   decode_binphilips2values(decode_manchester2bin(buffer));
		  state = (state + 1)%2;
		} // end if ( cont == 30 )

		

		if (cont > BIG_MAX )
		  {
		    // a very big time is the time between 2 frame
		    //printf("\n******* STATING NEW FRAME *************\n");
		    pointeur_buffer = 1;
		    buffer[0] = '1';
		  } // end if  (cont > BIG_MAX )
		
		
		fflush(stdout);
		
		state = (state + 1)%2;
		
	  } // end while
	return 0;
} // end main
