/**
 * remote.c
 * Pour voir les messages du noyau:
 * tail -f /var/log/messages
 *
 * dans /usr/src/rtai...
 * make dev -> creer les devices
 * ls -lsa /dev/rtf* -> imprime les drivers des devices
 **/
                                                                                                       
/* Importations */
#include <linux/kernel.h>
#include <linux/module.h>
#include <rtai.h>
#include <rtai_sched.h>

/* la valeur passee en parametre */
static int periode = 0;
                                                                                                       
/* le declaration des parametres */
MODULE_PARM(periode ,"l");

// creation de la tache
RT_TASK remote_task;

void remotePeriodicTask() 
{
  
  unsigned short in=0;
  unsigned short buffer=0;
  int cpt=0;
  
  while(1) 
    {
      // lire le port
      in=inb((0x378)+1) & 0x40;
      
      // on ecrit que si on a une nouvelle valeurs
      if (in!=buffer)
	{
	  // ecriture dans la fifo
  	  //rtf_put(0,(char *)&in,sizeof(in));
	  rtf_put(0,(char *)&cpt,sizeof(cpt));
	  buffer=in;
	  cpt=1;
	}
      else
	{
	  if (cpt==30){  rtf_put(0,(char *)&cpt,sizeof(cpt)); }
	  cpt++;
	}
      // permet de suspendre la tache, permet la preemption
      rt_task_wait_period();
    }// end while
}// end remotePeriodicTask

/* fonction d initialisation */
int init_module(void) 
{
                                                                                                       
  rt_printk("<1>Initialisation du module REMOTE (PERIODE[%d]) \n",periode);
  
  // parametres
  int arg = 1;
  int stack_size = 3000;
  int prio = 1;
  int use_fpu = 0;
                                                                                                       
  // creation de la fifo
  rtf_create(0,8192);
  rt_printk("<1>Fifo cree\n");
                                                                                                       
                                                                                                       
  // initialisation de la tache
  rt_task_init(&remote_task, remotePeriodicTask, arg, stack_size, prio, use_fpu, 0);
  // lance l execution periodique
  RTIME start_time = rt_get_time() + 3000;
  rt_task_make_periodic(&remote_task, start_time, nano2count(periode));
  start_rt_timer(nano2count(periode));
                                                                                                       
  // return sinon marche po
  return 0;
}// end init_module

/* fonction de dechargement */
void cleanup_module(void) 
{
                                                                                                       
  // arret du timer
  stop_rt_timer();
  
  rt_printk("<1>Dechargement du module REMOTE\n");
  
  // arrete les taches periodiques
  stop_rt_timer();
  rt_task_delete(&remote_task);
  rt_printk("<1>arret de la tache periodique\n");
  
  rtf_destroy(0);
  rt_printk("<1>destruction de la fifo\n");
  
}// end cleanup_module

